module Guard
  class Test < Plugin
    class Deprecator
      attr_accessor :options

      def self.warns_about_deprecated_options(options = {})
        new(options).warns_about_deprecated_options
      end

      def initialize(options = {})
        @options = options
      end

      def warns_about_deprecated_options
        _use_cmd_option
        _keep_failed_option
      end

      private

      def _use_cmd_option
        %w[bundler rubygems cli rvm drb spring zeus include].each do |option|
          next unless options.key?(option.to_sym)
          _deprecated("The :#{option} option is deprecated. Please customize the new :cmd option to fit your need.")
        end
      end

      def _keep_failed_option
        return unless options.key?(:keep_failed)
        _deprecated('The :keep_failed option is deprecated. Please set new :failed_mode option value to :keep instead. https://github.com/guard/guard-test#list-of-available-options')
      end

      def _deprecated(message)
        UI.warning %{Guard::RSpec DEPRECATION WARNING: #{message}}
      end

    end
  end
end
