module Guard
  class Test < Plugin
    module Inspectors

      class BaseInspector
        attr_accessor :options, :test_paths

        def initialize(options = {})
          @options = options
          @test_paths = @options[:test_paths]
        end

        def paths(paths)
          raise _abstract
        end

        def failed(locations)
          raise _abstract
        end

        def reload
          raise _abstract
        end

        private

        def _abstract
          'Must be implemented in subclass'
        end

        def _clean(paths)
          cleaned_paths = paths.dup

          paths.each do |path|
            if _test_folder?(path)
              cleaned_paths.delete(path)
              cleaned_paths += _check_test_files(path)
            else
              cleaned_paths.delete(path) unless _test_file?(path)
            end
          end

          cleaned_paths.uniq!
          cleaned_paths.compact!
          _clear_test_files_list
          puts "cleaned #{cleaned_paths}"

          cleaned_paths.sort - ['test/test_helper.rb']
        end

        def _test_folder?(path)
          paths = test_paths.join("|")
          path.match(%r{^\/?(#{paths})}) && !path.match(/\..+$/) && File.directory?(path)
        end

        def _test_file?(path)
          _test_files.include?(path)
        end

        def _test_files
          @_test_files ||= test_paths.collect { |path| _check_test_files(path) }.flatten
        end

        def _clear_test_files_list
          @_test_files = nil
        end

        def _check_test_files(path)
          Dir[File.join(path, '**', 'test_*.rb')] +
          Dir[File.join(path, '**', '*_test{s,}.rb')]
        end
      end

    end
  end
end
